import numpy
import matplotlib
import matplotlib.figure
from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas


def sigma_region_lower(mean, nsigma):
    # Get the lower bound nsigma * stdev of the poisson distribution
    return numpy.array((mean - nsigma * numpy.sqrt(mean))[::-1])

def sigma_region_upper(mean, nsigma):
    # Get the upper bound nsigma * stdev of the poisson distribution
    return numpy.array((mean + nsigma * numpy.sqrt(mean))[::-1])


matplotlib.rcParams.update({"font.size": 12.0,
        "axes.titlesize": 12.0,
        "axes.labelsize": 12.0,
        "xtick.labelsize": 12.0,
        "ytick.labelsize": 12.0,
        "legend.fontsize": 10.0,
        "figure.dpi": 300,
        "savefig.dpi": 300,
        "text.usetex": True,
        "path.simplify": True,
        "font.family": "serif"
})

fore_ifar = numpy.loadtxt('./Sec_5_Figure_4b_pycbc_ifars.dat',
                          dtype=numpy.float64)
fg_time = numpy.loadtxt('./Sec_5_Figure_4b_pycbc_fg_time.dat',
                          dtype=numpy.float64)

# Sort IFARs into ascending order
sortifar = numpy.argsort(fore_ifar)
fore_ifar = fore_ifar[sortifar]

# Get the cummulative number with IFAR greater than or equal to
fore_cumnum = numpy.arange(len(fore_ifar), 0, -1)

# Get expected number from foreground time
expected_ifar = numpy.logspace(-4., 4., num=1000, base=10.0)
expected_cumnum = fg_time / expected_ifar
plotifar = expected_ifar[::-1]

# make figure
fig = matplotlib.figure.Figure(figsize=[6.49606299, 4.01478772])
FigureCanvas(fig)
axes = fig.add_axes((0.1, 0.12, .875, .80))
axes.tick_params(axis='both', which='minor')
axes.grid(axis='both', linestyle=':', c='k')
axes.loglog()

one_sig = axes.fill_between(plotifar,
                            sigma_region_lower(expected_cumnum, 1.),
                            sigma_region_upper(expected_cumnum, 1.),
                            alpha=0.5, facecolor=[0.25, 0.25, 0.25],
                            label=r'$\pm 1 \sigma$')
two_sig = axes.fill_between(plotifar,
                            sigma_region_lower(expected_cumnum, 2.),
                            sigma_region_upper(expected_cumnum, 2.),
                            alpha=0.5, facecolor=[0.5, 0.5, 0.5],
                            label=r'$\pm 2 \sigma$')
thr_sig = axes.fill_between(plotifar,
                            sigma_region_lower(expected_cumnum, 3.),
                            sigma_region_upper(expected_cumnum, 3.),
                            alpha=0.5, facecolor=[0.75, 0.75, 0.75],
                            label=r'$\pm 3 \sigma$')

# Plot the expected background
back_line = axes.plot(expected_ifar, expected_cumnum, 'k--', linewidth=1,
                      label=r'Expected background')

# Plot the foreground triggers
fore_line = axes.step(fore_ifar, fore_cumnum, color='k', linewidth=2,
                      label='Observed')

# Format figure
axes.set_ybound(0.1, 0.1 * max(expected_cumnum))
axes.set_xbound(1600 * min(fore_ifar), 2 * expected_ifar.max())
axes.set_ylabel('Cumulative Number of Events')
axes.set_xlabel('Inverse False-Alarm Rate (yr)')
axes.legend((fore_line[0], back_line[0], one_sig, two_sig, thr_sig),
            ("Observed", r"Expected background", r"$\pm\sigma$", r"$\pm 2\sigma$", r"$\pm 3\sigma$"),
            loc="upper right")

if fig is not None:
    try:
        fig.tight_layout(rect=(0.03, 0.02, 0.98, 0.98))
    except ValueError:
        pass

# Save figure
fig.savefig('./Sec_5_Figure_4b.pdf')

